@@ -1,21 +1,24 @@
+use std::mem::replace;
+
 use crate::links::Links;
 use crate::non_max::NonMax;
 use crate::span::{usize_to_index, Index};
 use crate::{Kind, Span, Tree, TreeError};
 
-#[derive(Debug, Clone, Copy)]
-struct Parent {
-    id: NonMax,
-}
-
 /// The identifier of a node as returned by functions such as
 /// [TreeBuilder::open] or [TreeBuilder::token].
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[repr(transparent)]
+pub struct Id(pub(crate) NonMax);
+
+/// The identifier of a node as returned by functions such as
+/// [TreeBuilder::checkpoint].
 ///
 /// This can be used as a checkpoint in [TreeBuilder::close_at], and a
 /// checkpoint can be fetched up front from [TreeBuilder::checkpoint].
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, PartialOrd, Ord)]
 #[repr(transparent)]
-pub struct Id(pub(crate) NonMax);
+pub struct Checkpoint(pub(crate) NonMax);
 
 impl Id {
     pub(crate) const fn new(id: NonMax) -> Self {
@@ -60,7 +63,12 @@ pub struct TreeBuilder<T> {
     /// Data in the tree being built.
     tree: Tree<T>,
     /// References to parent nodes of the current node being constructed.
-    parents: Vec<Parent>,
+    parents: Vec<NonMax>,
+    /// What checkpoints in use refer to.
+    checkpoints: Vec<NonMax>,
+    /// The last checkpoint handed out. This will be invalidated once the tree
+    /// is modified.
+    checkpoint: Option<NonMax>,
     /// Reference to last sibling inserted.
     sibling: Option<NonMax>,
     /// The current cursor.
@@ -107,6 +115,8 @@ impl<T> TreeBuilder<T> {
         TreeBuilder {
             tree: Tree::new(),
             parents: Vec::new(),
+            checkpoints: Vec::new(),
+            checkpoint: None,
             sibling: None,
             cursor: 0,
         }
@@ -138,7 +148,7 @@ impl<T> TreeBuilder<T> {
     /// ```
     pub fn open(&mut self, data: T) -> Result<Id, TreeError> {
         let id = self.insert(data, Kind::Node, Span::point(self.cursor))?;
-        self.parents.push(Parent { id });
+        self.parents.push(id);
         Ok(Id(id))
     }
 
@@ -175,16 +185,19 @@ impl<T> TreeBuilder<T> {
             None => return Err(TreeError::CloseError),
         };
 
-        self.sibling = Some(head.id);
+        self.sibling = Some(head);
 
-        if let Some(parent) = self.parents.last().copied() {
-            if let Some(node) = self.tree.get_mut(head.id) {
-                let end = node.span.end;
-
-                if let Some(parent) = self.tree.get_mut(parent.id) {
-                    parent.span.end = end;
-                }
-            }
+        if let Some(&parent) = self.parents.last() {
+            let node = self
+                .tree
+                .get_mut(head)
+                .ok_or(TreeError::MissingNode(Id(head)))?;
+            let end = node.span.end;
+            let parent = self
+                .tree
+                .get_mut(parent)
+                .ok_or(TreeError::MissingNode(Id(parent)))?;
+            parent.span.end = end;
         }
 
         Ok(())
@@ -214,7 +227,7 @@ impl<T> TreeBuilder<T> {
         if len > 0 {
             self.cursor = usize_to_index(len)
                 .and_then(|len| self.cursor.checked_add(len))
-                .ok_or(TreeError::IdOverflow)?;
+                .ok_or(TreeError::Overflow)?;
             self.tree.span_mut().end = self.cursor;
         }
 
@@ -262,10 +275,16 @@ impl<T> TreeBuilder<T> {
     /// assert_eq!(tree, expected);
     /// # Ok(()) }
     /// ```
-    pub fn checkpoint(&self) -> Result<Id, TreeError> {
-        Ok(Id(
-            NonMax::new(self.tree.len()).ok_or(TreeError::IdOverflow)?
-        ))
+    pub fn checkpoint(&mut self) -> Result<Checkpoint, TreeError> {
+        if let Some(c) = self.checkpoint {
+            return Ok(Checkpoint(c));
+        }
+
+        let id = NonMax::new(self.tree.len()).ok_or(TreeError::Overflow)?;
+        let c = NonMax::new(self.checkpoints.len()).ok_or(TreeError::Overflow)?;
+        self.checkpoints.push(id);
+        self.checkpoint = Some(c);
+        Ok(Checkpoint(c))
     }
 
     /// Insert a node that wraps from the given checkpointed location.
@@ -376,10 +395,16 @@ impl<T> TreeBuilder<T> {
     /// assert_eq!(tree, expected);
     /// # Ok(()) }
     /// ```
-    pub fn close_at(&mut self, id: Id, data: T) -> Result<Id, TreeError> {
-        let next_id = NonMax::new(self.tree.len()).ok_or(TreeError::IdOverflow)?;
+    pub fn close_at(&mut self, c: Checkpoint, data: T) -> Result<Id, TreeError> {
+        let checkpoint_id = self
+            .checkpoints
+            .get_mut(c.0.get())
+            .ok_or(TreeError::MissingCheckpoint(c))?;
+        let id = *checkpoint_id;
 
-        let links = match self.tree.get_mut(id.0) {
+        let next_id = NonMax::new(self.tree.len()).ok_or(TreeError::Overflow)?;
+
+        let links = match self.tree.get_mut(id) {
             Some(links) => links,
             None => {
                 let id = self.insert(data, Kind::Node, Span::point(self.cursor))?;
@@ -388,81 +413,61 @@ impl<T> TreeBuilder<T> {
             }
         };
 
-        // We attempt to restructure the current node, unless it is an immediate
-        // and sole sibling - which it is if it was just inserted into
-        // next.sibling.
-        //
-        // The primary purpose of restructuring is to calculate the span of the
-        // range of nodes we are wrapping.
-        let needs_restructuring = self.sibling != Some(id.0);
+        let parent = replace(&mut links.parent, Some(next_id));
+        let prev = replace(&mut links.prev, None);
+
+        // Restructuring is necessary to calculate the full span of the newly
+        // inserted node and update parent references to point to the newly
+        // inserted node.
+        let (last, span) = if self.sibling != Some(id) {
+            let next = links.next.ok_or(TreeError::CloseAtError)?;
+            let start = links.span.start;
+            let (end, last) = restructure_close_at(&mut self.tree, self.sibling, next_id, next)?;
+            (Some(last), Span::new(start, end))
+        } else {
+            (None, links.span)
+        };
 
         let added = Links {
             data,
             kind: Kind::Node,
-            span: links.span,
-            parent: links.parent,
+            span,
+            prev,
+            parent,
             next: None,
-            first: Some(id.0),
+            first: Some(id),
+            last,
         };
 
-        links.parent = Some(next_id);
-        let start = links.span.start;
-        let links_next = links.next;
+        if let Some(parent) = parent.and_then(|id| self.tree.get_mut(id)) {
+            if parent.first == Some(id) {
+                parent.first = Some(next_id);
+            }
 
-        self.tree.push(added);
-
-        if needs_restructuring {
-            self.restructure_close_at(next_id, start, links_next)?;
+            if parent.last == Some(id) {
+                parent.last = Some(next_id);
+            }
         }
 
-        // The current sibling is the newly replaced node in the tree.
-        self.sibling = Some(next_id);
+        if let Some(prev) = prev.and_then(|id| self.tree.get_mut(id)) {
+            prev.next = Some(next_id);
+        }
 
         // If we're replacing the first node of the tree, the newly inserted
         // node should be set as the first node.
-        let first = self.tree.first_mut();
-
-        if Some(id.0) == *first {
+        if self.tree.first_id() == Some(id) {
+            let (first, _) = self.tree.links_mut();
             *first = Some(next_id);
         }
 
+        // Accounting.
+        self.tree.push(added);
+        self.checkpoint = None;
+        self.sibling = Some(next_id);
+        *checkpoint_id = next_id;
         Ok(Id(next_id))
     }
 
-    // Adjust span to encapsulate all children and check that we just inserted
-    // the checkpointed node in the right location which should be the tail
-    // sibling of the replaced node.
-    fn restructure_close_at(
-        &mut self,
-        id: NonMax,
-        start: Index,
-        mut removed_next: Option<NonMax>,
-    ) -> Result<(), TreeError> {
-        let mut last = None;
-
-        while let Some((next, next_id)) =
-            removed_next.and_then(|id| Some((self.tree.get_mut(id)?, id)))
-        {
-            next.parent = Some(id);
-            last = Some((next.span.end, next_id));
-            removed_next = next.next;
-        }
-
-        let (end, end_id) = last.ok_or(TreeError::CloseAtError)?;
-
-        let sibling = self.sibling.ok_or(TreeError::CloseAtError)?;
-
-        if sibling != end_id {
-            return Err(TreeError::CloseAtError);
-        }
-
-        if let Some(node) = self.tree.get_mut(id) {
-            node.span = Span::new(start, end);
-        }
-
-        Ok(())
-    }
-
     /// Build a [Tree] from the current state of the builder.
     ///
     /// This requires the stack in the builder to be empty. Otherwise a
@@ -523,32 +528,46 @@ impl<T> TreeBuilder<T> {
 
     /// Insert a new node.
     fn insert(&mut self, data: T, kind: Kind, span: Span) -> Result<NonMax, TreeError> {
-        let new = NonMax::new(self.tree.len()).ok_or(TreeError::IdOverflow)?;
+        let new = NonMax::new(self.tree.len()).ok_or(TreeError::Overflow)?;
 
+        self.checkpoint = None;
         let prev = std::mem::replace(&mut self.sibling, None);
-        let parent = self.parents.last();
+        let parent = self.parents.last().copied();
 
         self.tree.push(Links {
             data,
             kind,
             span,
-            parent: parent.map(|p| p.id),
+            parent,
+            prev,
             next: None,
             first: None,
+            last: None,
         });
 
+        if let Some(id) = parent {
+            if let Some(node) = self.tree.links_at_mut(id) {
+                if node.first.is_none() {
+                    node.first = Some(new);
+                }
+
+                node.last = Some(new);
+                node.span.end = span.end;
+            }
+        } else {
+            let (first, last) = self.tree.links_mut();
+
+            if first.is_none() {
+                *first = Some(new);
+            }
+
+            *last = Some(new);
+        }
+
         if let Some(node) = prev.and_then(|id| self.tree.links_at_mut(id)) {
             node.next = Some(new);
         }
 
-        if let Some(node) = parent.and_then(|p| self.tree.links_at_mut(p.id)) {
-            if node.first.is_none() {
-                node.first = Some(new);
-            }
-
-            node.span.end = span.end;
-        }
-
         Ok(new)
     }
 }
@@ -558,3 +577,31 @@ impl<T> Default for TreeBuilder<T> {
         Self::new()
     }
 }
+
+// Adjust span to encapsulate all children and check that we just inserted the
+// checkpointed node in the right location which should be the tail sibling of
+// the replaced node.
+fn restructure_close_at<T>(
+    tree: &mut Tree<T>,
+    sibling: Option<NonMax>,
+    parent_id: NonMax,
+    next: NonMax,
+) -> Result<(Index, NonMax), TreeError> {
+    let mut links = tree.get_mut(next).ok_or(TreeError::MissingNode(Id(next)))?;
+    let mut last = (links.span.end, next);
+    links.parent = Some(parent_id);
+
+    while let Some(next) = links.next {
+        links = tree.get_mut(next).ok_or(TreeError::MissingNode(Id(next)))?;
+        last = (links.span.end, next);
+        links.parent = Some(parent_id);
+    }
+
+    let sibling = sibling.ok_or(TreeError::CloseAtError)?;
+
+    if sibling != last.1 {
+        return Err(TreeError::CloseAtError);
+    }
+
+    Ok(last)
+}
