@@ -1,8 +1,29 @@
-use crate::links::Links;
-use crate::non_max::NonMaxUsize;
-use crate::{Kind, Node, WithoutTokens};
+use std::iter::FusedIterator;
 
-/// Iterator over the children of a node or tree.
+use crate::{links::Links, non_max::NonMax, Kind, Node, SkipTokens};
+
+/// An iterator that iterates over the [Node::next] elements of a node. This is
+/// typically used for iterating over the children of a tree.
+///
+/// Note that this iterator also implements [Default], allowing it to
+/// effectively create an empty iterator in case a particular sibling is not
+/// available:
+///
+/// ```
+/// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+/// let mut tree = syntree::tree! {
+///     "root" => {
+///         "child1" => {
+///             "child2"
+///         },
+///         "child3"
+///     }
+/// };
+///
+/// let mut it = tree.first().and_then(|n| n.last()).map(|n| n.children()).unwrap_or_default();
+/// assert_eq!(it.next().map(|n| *n.value()), None);
+/// # Ok(()) }
+/// ```
 ///
 /// See [Tree::children][crate::Tree::children] or [Node::children].
 ///
@@ -27,31 +48,46 @@ use crate::{Kind, Node, WithoutTokens};
 ///     ["root", "root2"]
 /// );
 ///
+/// assert_eq!(
+///     tree.children().rev().map(|n| *n.value()).collect::<Vec<_>>(),
+///     ["root2", "root"]
+/// );
+///
 /// let root = tree.first().ok_or("missing root node")?;
 ///
 /// assert_eq!(
 ///     root.children().map(|n| *n.value()).collect::<Vec<_>>(),
 ///     ["child1", "child3"]
 /// );
+///
+/// assert_eq!(
+///     root.children().rev().map(|n| *n.value()).collect::<Vec<_>>(),
+///     ["child3", "child1"]
+/// );
 /// # Ok(()) }
 /// ```
 pub struct Children<'a, T> {
     tree: &'a [Links<T>],
-    node: Option<NonMaxUsize>,
+    first: Option<NonMax>,
+    last: Option<NonMax>,
 }
 
 impl<'a, T> Children<'a, T> {
     /// Construct a new child iterator.
-    pub(crate) fn new(tree: &'a [Links<T>], node: Option<NonMaxUsize>) -> Self {
-        Self { tree, node }
+    pub(crate) const fn new(
+        tree: &'a [Links<T>],
+        first: Option<NonMax>,
+        last: Option<NonMax>,
+    ) -> Self {
+        Self { tree, first, last }
     }
 
-    /// Construct a [WithoutTokens] iterator from the remainder of this
+    /// Construct a [SkipTokens] iterator from the remainder of this
     /// iterator. This filters out [Kind::Token] elements.
     ///
-    /// See [WithoutTokens] for documentation.
-    pub fn without_tokens(self) -> WithoutTokens<Self> {
-        WithoutTokens::new(self)
+    /// See [SkipTokens] for documentation.
+    pub const fn skip_tokens(self) -> SkipTokens<Self> {
+        SkipTokens::new(self)
     }
 
     /// Get the next node from the iterator. This advances past all non-node
@@ -72,17 +108,21 @@ impl<'a, T> Children<'a, T> {
     /// };
     ///
     /// let mut it = tree.children();
-    /// assert_eq!(it.next_node().map(|n| *n.value()), Some("child1"));
-    /// assert_eq!(it.next().map(|n| *n.value()), Some("t2"));
+    /// let mut out = Vec::new();
+    ///
+    /// while let Some(n) = it.next_node() {
+    ///     out.push(*n.value());
+    /// }
+    ///
+    /// assert_eq!(out, ["child1", "child2", "child3"]);
     /// # Ok(()) }
     /// ```
     pub fn next_node(&mut self) -> Option<Node<'a, T>> {
         loop {
-            let node = self.tree.get(self.node?.get())?;
-            self.node = node.next;
+            let node = self.next()?;
 
-            if matches!(node.kind, Kind::Node) {
-                return Some(Node::new(node, self.tree));
+            if matches!(node.kind(), Kind::Node) {
+                return Some(node);
             }
         }
     }
@@ -92,12 +132,32 @@ impl<'a, T> Iterator for Children<'a, T> {
     type Item = Node<'a, T>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let node = self.tree.get(self.node?.get())?;
-        self.node = node.next;
+        let first = self.first.take()?;
+        let node = self.tree.get(first.get())?;
+
+        if first != self.last? {
+            self.first = node.next;
+        }
+
         Some(Node::new(node, self.tree))
     }
 }
 
+impl<'a, T> DoubleEndedIterator for Children<'a, T> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        let last = self.last.take()?;
+        let node = self.tree.get(last.get())?;
+
+        if last != self.first? {
+            self.last = node.prev;
+        }
+
+        Some(Node::new(node, self.tree))
+    }
+}
+
+impl<'a, T> FusedIterator for Children<'a, T> {}
+
 impl<'a, T> Clone for Children<'a, T> {
     fn clone(&self) -> Self {
         *self
@@ -105,3 +165,13 @@ impl<'a, T> Clone for Children<'a, T> {
 }
 
 impl<'a, T> Copy for Children<'a, T> {}
+
+impl<'a, T> Default for Children<'a, T> {
+    fn default() -> Self {
+        Self {
+            tree: &[],
+            first: None,
+            last: None,
+        }
+    }
+}
