@@ -1,6 +1,8 @@
 use std::error::Error;
 use std::fmt;
 
+use crate::{Checkpoint, Id};
+
 /// Errors raised while building a tree.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 #[non_exhaustive]
@@ -67,18 +69,16 @@ pub enum TreeError {
     /// # Ok(()) }
     /// ```
     CloseAtError,
-    /// Cursor overflowed.
+    /// Numerical overflow.
     ///
     /// This only happens under extreme circumstances or if a feature is enabled
     /// which narrows the width of an identifier to the degree that this error
     /// is easier to accomplish.
-    CursorOverflow,
-    /// Identifier overflowed.
-    ///
-    /// This only happens under extreme circumstances or if a feature is enabled
-    /// which narrows the width of an identifier to the degree that this error
-    /// is easier to accomplish.
-    IdOverflow,
+    Overflow,
+    /// The given checkpoint doesn't exist.
+    MissingCheckpoint(Checkpoint),
+    /// The node of the given id is missing.
+    MissingNode(Id),
 }
 
 impl Error for TreeError {}
@@ -98,11 +98,14 @@ impl fmt::Display for TreeError {
                     "trying to close a node which is not a sibling of the checkpoint being closed"
                 )
             }
-            TreeError::CursorOverflow => {
-                write!(f, "index cursor overflow")
+            TreeError::Overflow => {
+                write!(f, "numerical overflow")
             }
-            TreeError::IdOverflow => {
-                write!(f, "identifier overflow")
+            TreeError::MissingCheckpoint(c) => {
+                write!(f, "missing checkpoint with id `{}`", c.0.get())
+            }
+            TreeError::MissingNode(id) => {
+                write!(f, "missing node with id `{}`", id.0.get())
             }
         }
     }
