@@ -3,7 +3,7 @@
 use crate::non_max::NonMax;
 use crate::{Kind, Span};
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct Links<T> {
     /// The data in the node.
     pub(crate) data: T,
@@ -11,10 +11,15 @@ pub struct Links<T> {
     pub(crate) kind: Kind,
     /// Span of the node.
     pub(crate) span: Span,
-    /// Parent node.
+    /// Parent node. These exists because they are needed when performing range
+    /// queries such as [Tree::node_with_range][crate::Tree::node_with_range].
     pub(crate) parent: Option<NonMax>,
+    /// The previous node.
+    pub(crate) prev: Option<NonMax>,
     /// Next sibling node.
     pub(crate) next: Option<NonMax>,
     /// First child node.
     pub(crate) first: Option<NonMax>,
+    /// Last child node.
+    pub(crate) last: Option<NonMax>,
 }
