@@ -5,7 +5,7 @@ use std::ops::Range;
 use crate::links::Links;
 use crate::non_max::NonMax;
 use crate::tree::Kind;
-use crate::{Ancestors, Id, Siblings, Span, Walk, WalkEvents};
+use crate::{Ancestors, Children, Id, Siblings, Span, Walk, WalkEvents};
 
 /// A node in the tree.
 pub struct Node<'a, T> {
@@ -14,8 +14,8 @@ pub struct Node<'a, T> {
 }
 
 impl<'a, T> Node<'a, T> {
-    pub(crate) const fn new(node: &'a Links<T>, tree: &'a [Links<T>]) -> Self {
-        Self { links: node, tree }
+    pub(crate) const fn new(links: &'a Links<T>, tree: &'a [Links<T>]) -> Self {
+        Self { links, tree }
     }
 
     /// Get the identifier of the current node.
@@ -211,14 +211,14 @@ impl<'a, T> Node<'a, T> {
     ///
     /// See [Siblings] for documentation.
     pub fn siblings(&self) -> Siblings<'a, T> {
-        Siblings::new(Some(*self))
+        Siblings::new(self.tree, self.links)
     }
 
     /// Get an iterator over the children of this node.
     ///
-    /// See [Siblings] for documentation.
-    pub fn children(&self) -> Siblings<'a, T> {
-        Siblings::new(self.first())
+    /// See [Children] for documentation.
+    pub fn children(&self) -> Children<'a, T> {
+        Children::new(self.tree, self.links.first, self.links.last)
     }
 
     /// Walk the subtree forward starting with the first child of the current
@@ -226,7 +226,7 @@ impl<'a, T> Node<'a, T> {
     ///
     /// See [Walk] for documentation.
     pub fn walk(&self) -> Walk<'a, T> {
-        Walk::new(self.first())
+        Walk::new(self.tree, self.links.first)
     }
 
     /// Walk the node forwards in a depth-first fashion emitting events
@@ -234,7 +234,7 @@ impl<'a, T> Node<'a, T> {
     ///
     /// See [WalkEvents] for documentation.
     pub fn walk_events(&self) -> WalkEvents<'a, T> {
-        WalkEvents::new(self.first())
+        WalkEvents::new(self.tree, self.links.first)
     }
 
     /// Get immediate parent to this node.
@@ -272,6 +272,38 @@ impl<'a, T> Node<'a, T> {
         self.node_at(self.links.parent?)
     }
 
+    /// Get the previous sibling.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let tree = syntree::tree! {
+    ///     "root" => {
+    ///         "number" => {
+    ///             ("lit", 5)
+    ///         },
+    ///         "ident" => {
+    ///             ("lit", 3)
+    ///         }
+    ///     }
+    /// };
+    ///
+    /// let number = tree.first().and_then(|n| n.first()).ok_or("missing number")?;
+    /// assert_eq!(*number.value(), "number");
+    /// assert!(number.prev().is_none());
+    ///
+    /// let ident = number.next().ok_or("missing ident")?;
+    /// assert_eq!(*ident.value(), "ident");
+    ///
+    /// let number = ident.prev().ok_or("missing number")?;
+    /// assert_eq!(*number.value(), "number");
+    /// # Ok(()) }
+    /// ```
+    pub fn prev(self) -> Option<Node<'a, T>> {
+        self.node_at(self.links.prev?)
+    }
+
     /// Get the next sibling.
     ///
     /// # Examples
@@ -334,6 +366,37 @@ impl<'a, T> Node<'a, T> {
         self.node_at(self.links.first?)
     }
 
+    /// Get the last child node.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
+    /// let tree = syntree::tree! {
+    ///     "root" => {
+    ///         "number" => {
+    ///             ("lit", 5)
+    ///         },
+    ///         "ident" => {
+    ///             ("lit", 3)
+    ///         }
+    ///     },
+    ///     "root2" => {
+    ///         ("whitespace", 5)
+    ///     }
+    /// };
+    ///
+    /// let root2 = tree.last().ok_or("missing root2")?;
+    /// assert_eq!(*root2.value(), "root2");
+    ///
+    /// let whitespace = root2.last().ok_or("missing whitespace")?;
+    /// assert_eq!(*whitespace.value(), "whitespace");
+    /// # Ok(()) }
+    /// ```
+    pub fn last(&self) -> Option<Node<'a, T>> {
+        self.node_at(self.links.last?)
+    }
+
     fn node_at(&self, id: NonMax) -> Option<Node<'a, T>> {
         let cur = self.tree.get(id.get())?;
 
@@ -352,6 +415,7 @@ where
         f.debug_struct("Node")
             .field("data", &self.links.data)
             .field("kind", &self.links.kind)
+            .field("span", &self.links.span)
             .finish()
     }
 }
@@ -369,7 +433,9 @@ where
     T: PartialEq,
 {
     fn eq(&self, other: &Self) -> bool {
-        self.links.data == other.links.data && self.links.kind == other.links.kind
+        self.links.data == other.links.data
+            && self.links.kind == other.links.kind
+            && self.links.span == other.links.span
     }
 }
 
